// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/components/ui/UIComponentRenderable.hpp"
#include "display/mesh/QuadMesh.hpp"


#define NEW_LABEL_CHAR_BEGIN 0x00
#define NEW_LABEL_CHAR_END 0xFF

namespace Dawn {
  struct UILabelChar {
    int32_t advanceX;
    int32_t advanceY;
    glm::vec2 bitmapSize;
    glm::vec2 bitmapPosition;
    float_t textureY;
  };

  struct UILabelFontDef {
    Texture texture;
    uint32_t fontSize;
    FT_Face *face;
    std::map<FT_ULong, struct UILabelChar> charStore;
  };

  struct UILabelPart {
    UILabelFontDef *fontDef;
    std::string text;
  };

  class UILabelNew : public UIComponentRenderable {
    protected:
      Mesh mesh;
      FontShaderBuffer shaderBuffer;

      Mesh testMesh;

      FT_Face face;
      FT_Face faceBold;
      FT_Face faceItalics;

      struct UILabelFontDef defFace;
      struct UILabelFontDef defFaceBold;
      struct UILabelFontDef defFaceItalics;

    public:
      UILabelNew(SceneItem *item);

      void onStart() override;
      std::vector<struct ShaderPassItem> getUIRenderPasses() override;
      float_t getWidth() override;
      float_t getHeight() override;
      float_t getContentWidth() override;
      float_t getContentHeight() override;
  };
}