// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "UIShader.hpp"

using namespace Dawn;

void UIShader::compile() {
  #if DAWN_OPENGL_GLSL
    this->compileShader(
      {
        { "aPos", 0 },
        { "aTexCoord", 1 }
      },
      // Vertex Shader
      "#version 330 core\n"
      "layout (location = 0) in vec3 aPos;\n"
      "layout (location = 1) in vec2 aTexCoord;\n"
      
      "layout (std140) uniform ub_UICanvas {\n"
        "mat4 u_View;\n"
        "mat4 u_Projection;\n"
      "};"

      "uniform mat4 u_Model;\n"

      "out vec2 o_TextCoord;\n"
      "void main() {\n"
        "gl_Position = u_Projection * u_View * u_Model * vec4(aPos, 1.0);\n"
        "o_TextCoord = vec2(aTexCoord.x, aTexCoord.y);\n"
      "}",

      // Fragment Shader
      "#version 330 core\n"

      "in vec2 o_TextCoord;\n"
      "out vec4 o_Color;\n"
      "uniform vec4 u_Color;\n"
      "uniform bool u_HasTexture;\n"
      "uniform sampler2D u_Text;\n"

      "void main() {\n"
        "if(u_HasTexture) {\n"
          "o_Color = texture(u_Text, o_TextCoord) * u_Color;\n"
          // "o_Color = u_Color;\n"
          // "o_Color.a = texture(u_Text, o_TextCoord).r;\n"
        "} else {\n"
          "o_Color = u_Color;"
        "}\n"
      "}\n"
    );
  #else
    #error Shader Type must be GLSL
  #endif

  this->paramModel = this->getParameterByName("u_Model");
  this->paramColor = this->getParameterByName("u_Color");
  this->paramTexture = this->getParameterByName("u_Text");
  this->paramHasTexture = this->getParameterByName("u_HasTexture");
  this->bufferUiCanvas = this->getBufferLocationByName("ub_UICanvas");
}